package com.massa.mrules.demos.insurance.batch.config;

import org.springframework.batch.core.Job;
import org.springframework.batch.core.JobParametersBuilder;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.ContextRefreshedEvent;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;

import com.massa.log.Log;
import com.massa.log.LogFactory;

/**
 * Configures beans that will schedule Spring batch Job executions.<br>
 */
@Configuration
@EnableScheduling
public class BatchSchedulingConfiguration implements ApplicationListener<ContextRefreshedEvent> {

	private static final Log LOG = LogFactory.getLog(BatchSchedulingConfiguration.class);

	@Autowired
	private JobLauncher jobLauncher;
	
	@Autowired
	@Qualifier("jobScheduler")
	private ThreadPoolTaskScheduler scheduler;

	@Autowired
	private ApplicationContext ctx;
	
	@Override
	public void onApplicationEvent(ContextRefreshedEvent event) {
		LOG.info("Scheduling job every 15 seconds.");
		scheduler.schedule(createTaskExecutorForJob(), new CronTrigger("*/15 * * * * *"));
	}

	private Runnable createTaskExecutorForJob() {
		return () -> {
			Job monJob = ctx.getBean("mrulesJob", Job.class);
			try {
				jobLauncher.run(monJob, new JobParametersBuilder().addLong("date", System.currentTimeMillis()).toJobParameters());
			} catch (Exception e) {
				LOG.error("Error while launching job.", e);
			}
		};
	}
	
	@Bean(value = "jobScheduler", destroyMethod = "shutdown")
    public ThreadPoolTaskScheduler jobScheduler() {
        // use the Spring ThreadPoolTaskScheduler
        ThreadPoolTaskScheduler scheduledExecutorService = new ThreadPoolTaskScheduler();
        // always set the poolsize
        scheduledExecutorService.setPoolSize(1);
        // for logging add a threadNamePrefix
        scheduledExecutorService.setThreadNamePrefix("JobScheduler-");
        // do not wait for completion of the task
        scheduledExecutorService.setWaitForTasksToCompleteOnShutdown(false);
        scheduledExecutorService.setAwaitTerminationSeconds(0);
        return scheduledExecutorService;
    }
}
